#-----------------------------------------------------------------------------
# NanoSoC Synopsys synthesis tcl file to be run with dc_shell
# A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
#
# Contributors
#
# Daniel Newbrook (d.newbrook@soton.ac.uk)
#
# Copyright (C) 2021-3, SoC Labs (www.soclabs.org)
#-----------------------------------------------------------------------------

set rtlPath $env(SOCLABS_PROJECT_DIR)/imp/ASIC/nanosoc/src/
set top_module nanosoc_chip_pads
supress_message = {ELAB-405}
#####
# Set search_path
#
# List locations where your standard cell libraries may be located
#
#####
set search_path [list . $search_path $env(PHYS_IP)/arm/tsmc/cln65lp/sc9_base_rvt/r0p0/sdb/ $env(PHYS_IP)/arm/tsmc/cln65lp/sc9_base_rvt/r0p0/db/ $env(SOCLABS_PROJECT_DIR)/memories/rf]
set search_path [concat $rtlPath $search_path]
######
# Set Target Library
#
# Set a default target library for Design Compiler to target when compiling a design
#
######
set target_library "sc9_cln65lp_base_rvt_tt_typical_max_1p20v_25c.db RF_LIB_tt_1p20v_1p20v_25c.db"

######
# Set Link Library
#
# Set a default link library for Design Compiler to target when compiling a design
#
######
set link_library sc9_cln65lp_base_rvt_tt_typical_max_1p20v_25c.db

#read_file {/home/dwn1c21/SoC-Labs/accelerator-project/imp/ASIC/nanosoc/src/} -autoread -recursive -format verilog -top $top_module

analyze -format sverilog -lib WORK -define POWER_PINS [glob  $env(SOCLABS_PROJECT_DIR)/imp/ASIC/nanosoc/src/*.sv]
analyze -format verilog -lib WORK -define POWER_PINS [glob  $env(SOCLABS_PROJECT_DIR)/imp/ASIC/nanosoc/src/*.v]
elaborate $top_module -lib WORK
current_design $top_module

# Link Design
link

create_clock XTAL1 -period 10 -waveform {0 5}
set_clock_latency 0.3 XTAL1
set_input_delay 1.0 -clock XTAL1 [all_inputs]
set_output_delay 0.7 -clock XTAL1 [all_outputs]
set_load 0.1 [all_outputs]
set_max_fanout 1 [all_inputs]
set_fanout_load 8 [all_outputs]

compile_ultra -exact_map

write -hierarchy -format verilog -output ./nanosoc_chip_pads.vm
redirect [format "%s%s%s" ./ $top_module _area.rep] { report_area }
redirect -append [format "%s%s%s" ./ $top_module _area.rep] { report_reference }
redirect [format "%s%s%s" ./ $top_module _power.rep] { report_power }
redirect [format "%s%s%s" ./ $top_module _timing.rep] \
  { report_timing -path full -max_paths 100 -nets -transition_time -capacitance -significant_digits 3 -nosplit}


exit