//-----------------------------------------------------------------------------
// customised example Cortex-M0 controller UART with file logging
// A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
//
// Contributors
//
// David Flynn (d.w.flynn@soton.ac.uk)
//
// Copyright (C) 2023, SoC Labs (www.soclabs.org)
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Abstract : FT1248 1-bit data off-chip interface (emulate FT232H device)
// and allows cmsdk_uart_capture testbench models to log ADP ip, op streams
//-----------------------------------------------------------------------------


module nanosoc_axi_stream_io_8_txd_from_file
  #(parameter TXDFILENAME = "txd.cmd",
    parameter VERBOSE = 0)
  (
  input  wire  aclk,
  input  wire  aresetn,
  input  wire  txd8_ready,
  output wire  txd8_valid,
  output wire  [7:0] txd8_data
  );


 //----------------------------------------------
 //-- File I/O
 //----------------------------------------------


   integer        fd;       // channel descriptor for cmd file input
   integer        ch;
`define EOF -1

   reg       valid;
   reg [7:0] data8;
   
   initial
     begin
       valid <= 0;
//       $timeformat(-9, 0, " ns", 14);
       fd= $fopen(TXDFILENAME,"r");
       if (fd == 0)
          $write("** %m : input file failed to open **\n");
       else begin
         @(posedge aresetn);
         ch =  $fgetc(fd);
         while (ch != `EOF) begin
           @(posedge aclk);
           data8 <= (ch & 8'hff);
           valid <= 1'b1;
           @(posedge aclk);
           while (txd8_ready == 1'b0)
             @(posedge aclk);
           valid <=0;
           ch =  $fgetc(fd);
         end
         $write("** %m : file closed after stream TX completed **\n");
         $fclose(fd);
         valid <= 0;
       end
     end

assign txd8_valid = valid;
assign txd8_data = data8;

endmodule
